#!/usr/bin/python

import sys
import csv
import os
import urllib
from BeautifulSoup import BeautifulSoup

def get_whitelist_attrs(filename="whitelist_attrs.csv"):
    try:
        reader = csv.reader(open(filename))
    except:
        print "Error, couldn't open Attribute whitelist file."
        print "Defaulting to no allowed attributes."
        return [],[]
    allowed_attrs, transformed_attrs = [], []
    is_allowed = False
    for thing in reader:
        if thing[0] == "ALLOWED_ATTRS":
            is_allowed = True
            continue
        elif thing[0] == "TRANSFORMED_ATTRS":
            is_allowed = False
            continue
        if is_allowed:
            allowed_attrs.append(thing[0])
        else:
            transformed_attrs.append(thing[0])
    return allowed_attrs, transformed_attrs
    
def get_whitelist(filename="whitelist.csv"):
    """
    Read in the whitelist (a file, assumed to be in csv format), and
    return a tuple for each of the categories: ALLOWED_TAGS, and 
    SPECIAL_TAGS
    
    Also checks to see if each element in SPECIAL_TAGS has an 
    associated procedure in special_procs. If such a tag does
    not have an appropriate handler defined, then this program
    will print a warning message, and default to stripping out 
    said tag. (Same goes for SPECIAL_TAGS)
    """
    reader = csv.reader(open(filename))
    allowed_tags = []
    special_tags = []
    is_allowed = True
    for thing in reader:
        thing = thing[0]
        if thing == 'ALLOWED_TAGS':
            is_allowed = True
            continue
        elif thing == "SPECIAL_TAGS":
            is_allowed = False
            continue
        if is_allowed:
            allowed_tags.append(thing)
        else:
            if thing not in special_procs:
                print "Error - a handler has not been defined"
                print "        for {0}".format(thing)
                print "  Defaulting to stripping tag: {0}".format(thing)
            else:    
                special_tags.append(thing)        
            continue
    return allowed_tags, special_tags

def handle_href(attr_name, value, tag):
    clean_content = _sanitize_html(BeautifulSoup(tag.getText()))
    clean_href = _sanitize_html(BeautifulSoup(value))
    str1 = clean_content.getString()
    str2 = clean_href.getString()
    if str1 and str2:
        tag.parent.append(" " + str1 + " -> " + str2 + " ")
    elif str1:
        tag.parent.append(" " + str1 + " ")
    elif str2:
        tag.parent.append(" " + str2 + " ")
    del(tag[attr_name])
    
def is_img_ext(str):
    return str in ('.jpg', '.jpeg', '.png', '.gif', '.bmp', '.tiff')
    
def handle_img(tag, soup):
    def is_safe():
        """
        An img tag is safe if:
            1.) It is not part of a GET request
            2.) It points to a valid image extension, like .jpg, .png
        """
        try:
            if '?' in tag['src']:
                return False
            elif is_img_ext(os.path.splitext(tag['src'])[1]):
                return True
            else:
                return False
        except Exception:
            return False
        
    if is_safe():
        return tag
    else:
        return ""

def handle_tag(tag, soup):
    tag_name = tag.name.lower()
    if tag_name in special_procs:
        return special_procs[tag_name](tag, soup)
    else:
        tag.attrs = None
        return tag
    
def handle_attr(attr_name, value, tag):
    if attr_name in transformed_attrs:
        transformed_attrs[attr_name](attr_name, value, tag)
    
def sanitize_html(htmlfile):
    """
    Given an HTML file, keep only all tags/attributes that are 
    allowed by our whitelist.
    """
    return _sanitize_html(BeautifulSoup(htmlfile))
    
def _sanitize_html(soup):
    def keep_tag(tag_name):
        return (tag_name in allowed_tags or
                tag_name in special_tags)
    def keep_attr(attr_name):
        return (attr_name in allowed_attrs or 
                attr_name in transformed_attrs)
        
    allowed_tags, special_tags  = get_whitelist()
    allowed_attrs, transformed_attrs = get_whitelist_attrs()
    for tag in soup.findAll(True):
        for name, value in tag.attrs:
            if keep_attr(name):
                handle_attr(name,value,tag)
            else:
                del tag[name]

    for tag in soup.findAll(True):
        tag_name = tag.name.lower()
        if keep_tag(tag_name):
            tag.replaceWith(handle_tag(tag, soup))
        else:
            tag.extract()
    return soup
    
def main():
    file = sys.stdin.readlines()
    decoded = ""
    for line in file:
        decoded += urllib.unquote(line)
    sanitized = sanitize_html(decoded)
    print sanitized

special_procs = { 'img' : handle_img }
transformed_attrs = { 'href' : handle_href }    
    
if __name__ == '__main__':
    main()
